/* GCompris - numbers.js
 *
 * SPDX-FileCopyrightText: 2018 Rajat Asthana <rajatasthana4@gmail.com>
 *
 *   SPDX-License-Identifier: GPL-3.0-or-later
 */

function get() {
    return [
                { //level1
                    "bulbCount": 2,
                    "numbersToBeConverted": [1, 2, 3],
                    "enableHelp": true,
                    "bulbValueVisible": true
                },
                { //level 2
                    "bulbCount": 2,
                    "numbersToBeConverted": [1, 2, 3],
                    "enableHelp": false,
                    "bulbValueVisible": true
                },
                { //level3
                    "bulbCount": 4,
                    "numbersToBeConverted": [4, 9, 13, 15],
                    "enableHelp": true,
                    "bulbValueVisible": true
                },
                { //level4
                    "bulbCount": 4,
                    "numbersToBeConverted": [5, 10, 14, 7],
                    "enableHelp": false,
                    "bulbValueVisible": true
                },
                { //level5
                    "bulbCount": 8,
                    "numbersToBeConverted": [57, 152, 248, 239, 89, 101],
                    "enableHelp": true,
                    "bulbValueVisible": true
                },
                { //level6
                    "bulbCount": 8,
                    "numbersToBeConverted": [58, 153, 240, 236, 231, 255],
                    "enableHelp": false,
                    "bulbValueVisible": true
                },
                { //level7
                    "bulbCount": 4,
                    "numbersToBeConverted": [3, 12, 5, 10, 15, 11],
                    "enableHelp": false,
                    "bulbValueVisible": false
                },
                { //level8
                    "bulbCount": 8,
                    "numbersToBeConverted": [57, 152, 248, 239, 89, 101],
                    "enableHelp": false,
                    "bulbValueVisible": false
                }
    ];
}
